import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Point;
import java.util.List;

/**
 * This is just a subclass for all objects in the world. You can add every method you want here but you better don't delete or change the already existing methods.
 */
public abstract class Player extends Actor
{
    protected double exactX;
    protected double exactY;
    protected double velX;
    protected double velY;
    
    /**
     * Set the players location if possible.
     * 
     * @param x
     *      The new x location of the actor.
     * 
     * @param y
     *      The new y location of the actor.
     */
    public void setLocation(int x, int y) {
        if (getWorld().isBounded()) {
            if (SlideBarScrollingWorld.WORLD_WIDTH != 0) {
                if ((getExactX() - getWorld().getWidth()/2) - getWorld().getTotalXMovement() > SlideBarScrollingWorld.WORLD_WIDTH/2) {
                    x = (int) (getStartingPoint().getX() + SlideBarScrollingWorld.WORLD_WIDTH/2);
                }
                else if ((getExactX() - getWorld().getWidth()/2) - getWorld().getTotalXMovement() < -SlideBarScrollingWorld.WORLD_WIDTH/2) {
                    x = (int) (getStartingPoint().getX() - SlideBarScrollingWorld.WORLD_WIDTH/2);
                }
            }
            if (SlideBarScrollingWorld.WORLD_HEIGHT != 0) {
                if ((getExactY() - getWorld().getHeight()/2) - getWorld().getTotalYMovement() > SlideBarScrollingWorld.WORLD_HEIGHT/2) {
                    y = (int) (getStartingPoint().getY() + SlideBarScrollingWorld.WORLD_HEIGHT/2);
                }
                else if ((getExactY() - getWorld().getHeight()/2) - getWorld().getTotalYMovement() < -SlideBarScrollingWorld.WORLD_HEIGHT/2) {
                    y = (int) (getStartingPoint().getY() - SlideBarScrollingWorld.WORLD_HEIGHT/2);
                }
            }
        }
        exactX = x;
        exactY = y;
        super.setLocation(x, y);
    }
    /**
     * Set the players location if possible.
     * 
     * @param x
     *      The new x location of the actor.
     * 
     * @param y
     *      The new y location of the actor.
     */
    public void setLocation(double x, double y) {
        if (getWorld().isBounded()) {
            if (SlideBarScrollingWorld.WORLD_WIDTH != 0) {
                if ((getExactX() - getWorld().getWidth()/2) - getWorld().getTotalXMovement() > SlideBarScrollingWorld.WORLD_WIDTH/2) {
                    x = getStartingPoint().getX() + SlideBarScrollingWorld.WORLD_WIDTH/2;
                }
                else if ((getExactX() - getWorld().getWidth()/2) - getWorld().getTotalXMovement() < -SlideBarScrollingWorld.WORLD_WIDTH/2) {
                    x = getStartingPoint().getX() - SlideBarScrollingWorld.WORLD_WIDTH/2;
                }
            }
            if (SlideBarScrollingWorld.WORLD_HEIGHT != 0) {
                if ((getExactY() - getWorld().getHeight()/2) - getWorld().getTotalYMovement() > SlideBarScrollingWorld.WORLD_HEIGHT/2) {
                    y = getStartingPoint().getY() + SlideBarScrollingWorld.WORLD_HEIGHT/2;
                }
                else if ((getExactY() - getWorld().getHeight()/2) - getWorld().getTotalYMovement() < -SlideBarScrollingWorld.WORLD_HEIGHT/2) {
                    y = getStartingPoint().getY() - SlideBarScrollingWorld.WORLD_HEIGHT/2;
                }
            }
        }
        exactX = x;
        exactY = y;
        super.setLocation((int) x, (int) y);
    }
    
    /**
     * Get the players exact x location.
     * 
     * @return
     *      The players x location as a double value.
     */
    public double getExactX(){
        return exactX;
    }
    /**
     * Get the players exact y location.
     * 
     * @return
     *      The players y location as a double value.
     */
    public double getExactY(){
        return exactY;
    }
    
    /**
     * Get the players exact x location in the scrolling world.
     * 
     * @return
     *      The players x location related to the startingpoint of the scrolling world.
     */
    public double getScrollingX() {
        return getStartingPoint().getX() - getX();
    }
    /**
     * Get the players exact y location in the scrolling world.
     * 
     * @return
     *      The players y location related to the startingpoint of the scrolling world.
     */
    public double getScrollingY() {
        return getStartingPoint().getY() - getY();
    }
    
    /**
     * Get the location of the starting point (the world coordinates (0|0)).
     * 
     * @return
     *      The starting point of the scrolling world.
     */
    public java.awt.Point getStartingPoint() {
        return new Point(getWorld().getWidth()/2 + getWorld().getTotalXMovement(), getWorld().getHeight()/2 + getWorld().getTotalYMovement());
    }
    
    /**
     * Get the scrolling point of the actor, consisting the actors x and y scrolling location.
     * 
     * @return
     *      The players current scrolling point (x and y).
     */
    public java.awt.Point getScrollingPoint() {
        return new Point((int) getScrollingX(), (int) getScrollingY());
    }
    
    /**
     * Get the current world casted to MouseScrollingWorld.
     */
    public SlideBarScrollingWorld getWorld() {
        return ((SlideBarScrollingWorld) super.getWorld());
    }
}